#!/usr/bin/env python
#-*- coding: utf-8 -*-
#pylint: disable=
"""
File       : mknfold.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: Helper script to split input file into train/test
by using match/nfold values. Loop over input file and store
lines with match==random.randint(1,nfold) into test file. The
rest goes into train file.
"""

# system modules
import os
import sys
import gzip
import bz2
import random
import optparse

def fopen(fin, mode='r'):
    "Return file descriptor for given file"
    if  fin.endswith('.gz'):
        stream = gzip.open(fin, mode)
    elif  fin.endswith('.bz2'):
        stream = bz2.BZ2File(fin, mode)
    else:
        stream = open(fin, mode)
    return stream

class OptionParser():
    def __init__(self):
        "User based option parser"
        self.parser = optparse.OptionParser()
        self.parser.add_option("--fin", action="store", type="string",
            dest="fin", default="", help="Input file")
        self.parser.add_option("--match", action="store", type="int",
            dest="match", default=1, help="value to match nfold, default 1")
        self.parser.add_option("--nfold", action="store", type="int",
            dest="nfold", default=5, help="nfold value, default 5")
        self.parser.add_option("--headers", action="store_true",
            dest="headers", default=False, help="keep headers, default False")
    def get_opt(self):
        "Return list of options"
        return self.parser.parse_args()

def mknfold(fin, match=1, nfold=5, keep_headers=False):
    "Split input file into few"
    random.seed(123)
    headers = None
    with fopen(fin, 'r') as istream:
        with fopen(fin+'.train', 'w') as otrain:
            with fopen(fin+'.valid', 'w') as otest:
                if  keep_headers:
                    headers = istream.readline()
                if  headers:
                    otest.write(headers)
                    otrain.write(headers)
                for line in istream:
                    if  random.randint( 1 , nfold ) == match:
                        otest.write(line)
                    else:
                        otrain.write(line)

def main():
    "Main function"
    optmgr  = OptionParser()
    opts, _ = optmgr.get_opt()
    mknfold(opts.fin, opts.match, opts.nfold, opts.headers)

if __name__ == '__main__':
    main()
