#!/usr/bin/env Rscript
# clean-up session parameters
rm(list=ls())

library(doParallel)
library(parallel)

# define a dataframe with options we'll use in our program
out <- sprintf("out.csv")
train.file <- sprintf("data/new_train.csv")
test.file <- sprintf("data/new_test.csv")

opts <- data.frame(nround=35,   # number of rounds to run, used both in CV and training
                   out=out,     # out prediction file, empty value means no predictions
                   mem=16,       # default memory allocated for Java processes
                   seed=1,      # initial seed
                   clf="gbm",   # classifier name
                   shuffle=FALSE,     # shuffle the data
                   train=train.file,  # input train file
                   test=test.file     # input test files
                   )

# parse input arguments and assign proper values to out options
cmd_args <- commandArgs()
for (arg in cmd_args) {
    arg <- as.character(arg)
    match <- grep("^--help", arg)
    if (length(match) == 1) {
        usage <- sprintf("Usage:")
        for(name in names(opts)) {
            usage <- sprintf("%s --%s=%s", usage, name, opts[,name])
        }
        print(usage)
        quit(save="no")
    }
    for (name in names(opts)) {
        arg <- as.character(arg)
        aname <- sprintf("^--%s=", name)
        match <- grep(aname, arg)
        if (length(match) == 1) {
            val <- gsub(aname, "", arg)
            orig.val <- opts[,name]
            if(is.logical(orig.val)) opts[,name] <- as.logical(val)
            if(is.numeric(orig.val)) opts[,name] <- as.numeric(val)
            if(is.character(orig.val) || is.factor(orig.val)) opts[,name] <- as.character(val)
        }
    }
}
msg <- "opts:"
for(name in names(opts)) {
    msg <- sprintf("%s %s=%s", msg, name, opts[,name])
}
print("Parameters can be accessed via opts dataframe")
print(msg)

# increase Java heap size
# http://www.bramschoenmakers.nl/en/node/726
java_mem <- sprintf("-XX:-UseGCOverheadLimit -Xmx%sg", opts$mem)
options( java.parameters = java_mem )
print(sprintf("Java options: %s", java_mem))

ncores = detectCores()
cl <- makeCluster(ncores - 1)
registerDoParallel(cl, cores=ncores - 1)
print(sprintf("Number of cores: %s", ncores))
