#!/usr/bin/env python
#-*- coding: utf-8 -*-
#pylint: disable=
"""
File       : rank_avg.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: 
"""

# system modules
import os
import sys
import gzip
import argparse

# ML modules
import pandas as pd
import numpy as np
from scipy.stats import rankdata
from sklearn.preprocessing import MinMaxScaler

class OptionParser():
    def __init__(self):
        "User based option parser"
        self.parser = argparse.ArgumentParser(prog='PROG')
        self.parser.add_argument("--fin", action="store",
            dest="fin", default="", help="Input file")
        self.parser.add_argument("--fout", action="store",
            dest="fout", default="", help="Output file")

def ranks(fin, fout):
    files = [os.path.expanduser(f) for f in fin.split(',')]
    preds = pd.DataFrame()
    names = []
    for idx, fname in enumerate(files):
        if  fname.endswith('.gz'):
            fobj = gzip.open(fname, 'r')
        else:
            fobj = open(fname, 'r')
        df = pd.read_csv(fobj)
        if  idx == 0:
            preds['QuoteNumber'] = df.QuoteNumber
        pname = 'pred_%s' % idx
        preds[pname] = df.QuoteConversion_Flag
        rname = 'rank_%s' % idx
        preds['rank_%s' % idx] = rankdata(df.QuoteConversion_Flag)
        names.append(rname)
    preds['rank_avg'] = preds[names].mean(1)
    preds['final'] = MinMaxScaler().fit_transform(preds['rank_avg']).reshape(-1,1)
    print preds.head()
    data = {'QuoteNumber':preds['QuoteNumber'], 'QuoteConversion_Flag': preds['final']}
    sub = pd.DataFrame(data, columns=['QuoteNumber', 'QuoteConversion_Flag'])
    print("Write prediction to %s" % fout)
    sub.to_csv(fout, index=False)

def main():
    "Main function"
    optmgr  = OptionParser()
    opts = optmgr.parser.parse_args()
    ranks(opts.fin, opts.fout)

if __name__ == '__main__':
    main()
